from rest_framework import serializers

from django_global_log.models import DjangoGlobalLog


class LogSaveSerializer(serializers.ModelSerializer):
    class Meta:
        model = DjangoGlobalLog
        fields = "__all__"



import logging
import sys
import traceback
from timeit import default_timer as timer

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from django.utils.module_loading import import_string

from django_global_log.serializers import LogSaveSerializer
from django_global_log.utils import get_ip

logger = logging.getLogger("django")


class DjangoGlobalLogMiddleware(MiddlewareMixin):
    """全局日志中间件"""

    req_start_time_key = "_global_log_req_start_time"

    def process_request(self, request):
        setattr(request, self.req_start_time_key, timer())
        return None

    def process_response(self, request, response):
        try:
            self.log(request, response)
            logger.info("[DB Logging Success]")
        except Exception as err:
            msg = traceback.format_exc()
            logger.error("[DB Logging Failed] %s\n%s", str(err), msg)
        return response

    def log(self, request, response):
        req_end_time = timer()
        req_start_time = getattr(request, self.req_start_time_key, req_end_time)
        duration = int((req_end_time - req_start_time) * 1000)
        log_detail = {
            "operator": getattr(request.user, "pk", ""),
            "path": request.path,
            "detail": {
                "full_url": request.build_absolute_uri(),
                "params": request.GET,
                "resp_size": sys.getsizeof(response.content),
                "req_header": dict(request.headers),
            },
            "code": response.status_code,
            "duration": duration,
            "ip": get_ip(request),
        }
        self.save_log(log_detail)

    def save_log(self, detail):
        serializer = LogSaveSerializer(data=detail)
        serializer.is_valid(raise_exception=True)
        celery_log_func_string = getattr(settings, "GLOBAL_LOG_CELERY_FUNC", None)
        if celery_log_func_string is None:
            serializer.save()
        else:
            try:
                celery_log_func = import_string(celery_log_func_string)
                celery_log_func.delay(serializer.data)
            except ImportError:
                logger.error("[Celery Log Func Not Exists] %s", celery_log_func_string)
                serializer.save()



from django.db import migrations, models

import django_global_log.models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="DjangoGlobalLog",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "operator",
                    models.CharField(
                        blank=True, max_length=255, null=True, verbose_name="操作人"
                    ),
                ),
                ("path", models.TextField(verbose_name="请求地址")),
                ("code", models.SmallIntegerField(default=200, verbose_name="响应码")),
                (
                    "detail",
                    models.JSONField(
                        default=django_global_log.models.get_default_detail,
                        null=True,
                        verbose_name="详细信息",
                    ),
                ),
                ("duration", models.IntegerField(default=0, verbose_name="请求时长(ms)")),
                ("ip", models.GenericIPAddressField(verbose_name="IP地址")),
                (
                    "create_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="创建时间"),
                ),
            ],
            options={
                "verbose_name": "全局日志",
                "verbose_name_plural": "全局日志",
                "db_table": "django_global_log",
                "ordering": ["-id"],
            },
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _
from rest_framework import status


def get_default_detail():
    return {}


class DjangoGlobalLog(models.Model):
    """日志"""

    operator = models.CharField(_("操作人"), max_length=255, null=True, blank=True)
    path = models.TextField(_("请求地址"))
    code = models.SmallIntegerField(_("响应码"), default=status.HTTP_200_OK)
    detail = models.JSONField(_("详细信息"), default=get_default_detail, null=True)
    duration = models.IntegerField(_("请求时长(ms)"), default=0)
    ip = models.GenericIPAddressField(_("IP地址"))
    create_at = models.DateTimeField(_("创建时间"), auto_now_add=True)

    class Meta:
        db_table = "django_global_log"
        verbose_name = _("全局日志")
        verbose_name_plural = verbose_name
        ordering = ["-id"]

    def __str__(self):
        return "{}:{}".format(self.operator, self.path)



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from django_global_log.models import DjangoGlobalLog

USER_MODEL = get_user_model()


@admin.register(DjangoGlobalLog)
class LogAdmin(admin.ModelAdmin):
    list_display = ["id", "operator_name", "path", "code", "ip", "create_at"]
    list_filter = ["code"]

    @admin.display(description=_("操作人"))
    def operator_name(self, obj):
        try:
            return USER_MODEL.objects.get(pk=obj.operator).username
        except USER_MODEL.DoesNotExist:
            return None



def get_ip(request):
    if request.META.get("HTTP_X_REAL_IP"):
        return request.META.get("HTTP_X_REAL_IP")
    if request.META.get("HTTP_X_FORWARDED_FOR"):
        return request.META.get("HTTP_X_FORWARDED_FOR").replace(" ", "").split(",")[0]
    return request.META.get("REMOTE_ADDR")



#!/usr/bin/env python
# coding: utf-8
from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="django_global_log",
    version="0.0.2",
    author="Oren Zhang",
    url="https://www.oren.ink/",
    author_email="oren_zhang@outlook.com",
    description="A Log Tool for Django",
    packages=["django_global_log", "django_global_log.migrations"],
    install_requires=[
        "django>=3.0",
        "djangorestframework>=3.9.3",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
)
